@@ -1,6 +1,7 @@
 /* imlib.c
 
 Copyright 1999-2000 Tom Gilbert
+Copyright 2020      daltomi <daltomi@disroot.org>
 Copyright 2020      ideal <idealities@gmail.com>
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2020      spycapitan <spycapitan@protonmail.com>
@@ -37,13 +38,6 @@ int depth;
 Window root = 0;
 
 
-/* atexit register func. */
-static void uninit_x_and_imlib(void)
-{
-  if (disp) XCloseDisplay(disp);
-}
-
-
 void
 init_x_and_imlib(char *dispstr, int screen_num)
 {
@@ -56,8 +50,6 @@ init_x_and_imlib(char *dispstr, int screen_num)
       exit(EXIT_FAILURE);
    }
 
-   atexit(uninit_x_and_imlib);
-
    if (screen_num)
       scr = ScreenOfDisplay(disp, screen_num);
    else
