@@ -8,9 +8,14 @@ Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
+Copyright 2020      blockparole
+Copyright 2020      Cungsten Tarbide <ctarbide@tuta.io>
+Copyright 2020      daltomi <daltomi@disroot.org>
 Copyright 2020      Hinigatsu <hinigatsu@protonmail.com>
+Copyright 2020      nothub
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2020      spycapitan <spycapitan@protonmail.com>
+Copyright 2021      c0dev0id <sh+github@codevoid.de>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -35,6 +40,18 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "scrot.h"
 #include "options.h"
+#include <assert.h>
+
+
+/* atexit register func. */
+static void uninit_x_and_imlib(void)
+{
+  if (disp) {
+     XCloseDisplay(disp);
+     disp = NULL;
+  }
+}
+
 
 int
 main(int argc,
@@ -54,11 +71,13 @@ main(int argc,
 
   init_x_and_imlib(opt.display, 0);
 
+  atexit(uninit_x_and_imlib);
+
   if (!opt.output_file) {
     opt.output_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot.png");
     opt.thumb_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot-thumb.png");
   } else {
-    have_extension = scrot_have_file_extension(opt.output_file);
+    scrot_have_file_extension(opt.output_file, &have_extension);
   }
 
 
@@ -144,6 +163,14 @@ main(int argc,
     {
       if (opt.note != NULL)
         scrot_note_draw(image);
+
+      scrot_have_file_extension(opt.thumb_file, &have_extension);
+
+      if (!have_extension) {
+        imlib_context_set_image(thumbnail);
+        imlib_image_set_format("png");
+      }
+
       filename_thumb = im_printf(opt.thumb_file, tm, NULL, NULL, thumbnail);
       scrot_check_if_overwrite_file(&filename_thumb);
       gib_imlib_save_image_with_error_return(thumbnail, filename_thumb, &err);
@@ -180,12 +207,18 @@ scrot_do_delay(void)
   }
 }
 
-char* scrot_have_file_extension(char *filename)
+size_t scrot_have_file_extension(char const *filename, char **ext)
 {
-    char *ext = strrchr(filename, '.');
-    return (ext && (strlen(ext + 1) == 3)) ? ext : NULL;
+    *ext = strrchr(filename, '.');
+
+    if (*ext) {
+        return strlen(*ext);
+    }
+
+    return 0;
 }
 
+
 void scrot_check_if_overwrite_file(char **filename)
 {
   char *curfile = *filename;
@@ -194,41 +227,44 @@ void scrot_check_if_overwrite_file(char **filename)
 
   if (access(curfile, F_OK) == -1) return;
 
-  const int max_count = 999;
-  static int count = 0;
-  const char *extension = scrot_have_file_extension(curfile);
+  const size_t max_counter = 999;
+  size_t counter = 0;
+  char *ext = NULL;
+  size_t ext_len = 0;
   const size_t slen = strlen(curfile);
-  int nalloc = slen + 4 + 1; // _000 + NUL byte
+  size_t nalloc = slen + 4 + 1; // _000 + NUL byte
   char fmt[5];
   char *newname = NULL;
 
-  if (extension)
-    nalloc += 4; // .ext
+  ext_len = scrot_have_file_extension(curfile, &ext);
+
+  if (ext)
+    nalloc += ext_len; // .ext
 
   newname = calloc(nalloc, sizeof(char));
 
-  if (extension)
-    // exclude extension
-    memcpy(newname, curfile, slen - 4);
+  if (ext)
+    // exclude ext
+    memcpy(newname, curfile, slen - ext_len);
   else
     memcpy(newname, curfile, slen);
 
   do {
-    snprintf(fmt, 5, "_%03d", count++);
+    snprintf(fmt, 5, "_%03zu", counter++);
 
-    if (!extension) {
+    if (!ext) {
       strncpy(newname + slen, fmt, 5);
     } else {
-        strncpy((newname + slen)-4, fmt, 5);
-        strncat(newname, extension, 4);
+        strncpy((newname + slen) - ext_len, fmt, 5);
+        strncat(newname, ext, ext_len);
     }
       curfile = newname;
-  } while ((count < max_count) && (access(curfile, F_OK) == 0));
+  } while ((counter < max_counter) && (access(curfile, F_OK) == 0));
 
   free(*filename);
   *filename = newname;
 
-  if (count == max_count) {
+  if (counter == max_counter) {
     fprintf(stderr, "scrot can no longer generate new file names.\n"
                     "The last attempt is %s\n", newname);
     free(newname);
@@ -237,6 +273,32 @@ void scrot_check_if_overwrite_file(char **filename)
 }
 
 
+int scrot_match_window_class_name(Window target)
+{
+    assert(disp != NULL);
+
+    const int NOT_MATCH = 0;
+    const int MATCH = 1;
+    /* By default all class names match since window_class_name by default is NULL*/
+    int retval = MATCH;
+
+    if (opt.window_class_name == NULL) {
+        return retval;
+    }
+
+    XClassHint classHint;
+    retval = NOT_MATCH; // window_class_name != NULL, by default NOT_MATCH
+
+    if (XGetClassHint(disp, target, &classHint) != BadWindow) {
+        retval = options_cmp_window_class_name(classHint.res_class);
+        XFree(classHint.res_name);
+        XFree(classHint.res_class);
+    }
+
+    return retval;
+}
+
+
 void
 scrot_grab_mouse_pointer(const Imlib_Image image,
     const int ix_off, const int iy_off)
@@ -253,8 +315,9 @@ scrot_grab_mouse_pointer(const Imlib_Image image,
   pixels = (DATA32*)xcim->pixels;
 #else
   DATA32 data[width * height * 4];
-
-  for (size_t i = 0; i < (width * height); i++)
+  
+  size_t i;
+  for (i = 0; i < (width * height); i++)
     ((DATA32*)data)[i] = (DATA32)xcim->pixels[i];
 
   pixels = data;
@@ -265,7 +328,7 @@ scrot_grab_mouse_pointer(const Imlib_Image image,
   XFree(xcim);
 
   if (!imcursor) {
-     fprintf(stderr, "scrot_grab_mouse_pointer: Failed create image using data.");
+     fprintf(stderr, "scrot_grab_mouse_pointer: Failed create image using data.\n");
      exit(EXIT_FAILURE);
   }
 
@@ -310,7 +373,7 @@ scrot_exec_app(Imlib_Image image, struct tm *tm,
   if (ret == -1) {
     fprintf(stderr, "The child process could not be created: %s\n", strerror(errno));
   } else if (WEXITSTATUS(ret) == 127) {
-    fprintf(stderr, "scrot could not be executed the command: %s.\n", execstr);
+    fprintf(stderr, "scrot could not execute the command: %s.\n", execstr);
   }
 
   exit(0);
@@ -344,75 +407,30 @@ scrot_sel_and_grab_image(void)
   fd_set fdset;
   int count = 0, done = 0;
   int rx = 0, ry = 0, rw = 0, rh = 0, btn_pressed = 0;
-  int rect_x = 0, rect_y = 0, rect_w = 0, rect_h = 0;
-  Cursor cur_cross, cur_angle;
   Window target = None;
-  GC gc;
-  XGCValues gcval;
+  Status ret;
 
-  gcval.function = GXxor;
-  gcval.foreground = XWhitePixel(disp, 0);
-  gcval.background = XBlackPixel(disp, 0);
-  gcval.plane_mask = gcval.background ^ gcval.foreground;
-  gcval.subwindow_mode = IncludeInferiors;
-
-  if (opt.line_color != NULL) {
-    XColor clr_exact, clr_closest;
-    Status ret;
-
-    ret = XAllocNamedColor(disp, XDefaultColormap(disp, DefaultScreen(disp)),
-            opt.line_color, &clr_exact, &clr_closest);
-
-    if (ret == 0) {
-       free(opt.line_color);
-       fprintf(stderr, "Error allocate color:%s\n", strerror(BadColor));
-       exit(EXIT_FAILURE);
-    }
-
-    gcval.foreground = clr_exact.pixel;
-
-    free(opt.line_color);
-    opt.line_color = NULL;
-  }
+  scrot_selection_create();
 
   xfd = ConnectionNumber(disp);
   fdsize = xfd + 1;
 
-  cur_cross = XCreateFontCursor(disp, XC_cross);
-  cur_angle = XCreateFontCursor(disp, XC_lr_angle);
-
-  gc =
-    XCreateGC(disp, root,
-              GCFunction | GCForeground | GCBackground | GCSubwindowMode,
-              &gcval);
-
-  XSetLineAttributes(disp, gc, opt.line_width, opt.line_style, CapRound, JoinRound);
-
-  if ((XGrabPointer
-       (disp, root, False,
-        ButtonMotionMask | ButtonPressMask | ButtonReleaseMask, GrabModeAsync,
-        GrabModeAsync, root, cur_cross, CurrentTime) != GrabSuccess)) {
-    fprintf(stderr, "couldn't grab pointer:");
-    XFreeCursor(disp, cur_cross);
-    XFreeCursor(disp, cur_angle);
-    XFreeGC(disp, gc);
+  ret = XGrabKeyboard(disp, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
+  if (ret == AlreadyGrabbed) {
+    int attempts = 20;
+    struct timespec delay = {0, 50 * 1000L * 1000L};
+    do {
+      nanosleep(&delay, NULL);
+      ret = XGrabKeyboard(disp, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
+    } while (--attempts > 0 && ret == AlreadyGrabbed);
+  }
+  if (ret != GrabSuccess) {
+    fprintf(stderr, "failed to grab keyboard\n");
+    scrot_selection_destroy();
     exit(EXIT_FAILURE);
   }
 
 
-  if ((XGrabKeyboard
-       (disp, root, False, GrabModeAsync, GrabModeAsync,
-        CurrentTime) != GrabSuccess)) {
-    fprintf(stderr, "couldn't grab keyboard:");
-    XFreeCursor(disp, cur_cross);
-    XFreeCursor(disp, cur_angle);
-    XFreeGC(disp, gc);
-    exit(EXIT_FAILURE);
-  }
-
-  if (opt.freeze == 1)
-      XGrabServer(disp);
-
   while (1) {
     /* handle events here */
     while (!done && XPending(disp)) {
@@ -420,44 +438,14 @@ scrot_sel_and_grab_image(void)
       switch (ev.type) {
         case MotionNotify:
           if (btn_pressed) {
-            if (rect_w) {
-              /* re-draw the last rect to clear it */
-              XDrawRectangle(disp, root, gc, rect_x, rect_y, rect_w, rect_h);
-            } else {
-              /* Change the cursor to show we're selecting a region */
-              XChangeActivePointerGrab(disp,
-                                       ButtonMotionMask | ButtonReleaseMask,
-                                       cur_angle, CurrentTime);
-            }
-
-            rect_x = rx;
-            rect_y = ry;
-            rect_w = ev.xmotion.x - rect_x;
-            rect_h = ev.xmotion.y - rect_y;
-
-            if (rect_w == 0) ++rect_w;
-            if (rect_h == 0) ++rect_h;
-
-            if (rect_w < 0) {
-              rect_x += rect_w;
-              rect_w = 0 - rect_w;
-            }
-            if (rect_h < 0) {
-              rect_y += rect_h;
-              rect_h = 0 - rect_h;
-            }
-            /* draw rectangle */
-            XDrawRectangle(disp, root, gc, rect_x, rect_y, rect_w, rect_h);
-            XFlush(disp);
+              scrot_selection_motion_draw(rx, ry, ev.xbutton.x, ev.xbutton.y);
           }
           break;
         case ButtonPress:
           btn_pressed = 1;
           rx = ev.xbutton.x;
           ry = ev.xbutton.y;
-          target =
-            scrot_get_window(disp, ev.xbutton.subwindow, ev.xbutton.x,
-                             ev.xbutton.y);
+          target = scrot_get_window(disp, ev.xbutton.subwindow, ev.xbutton.x, ev.xbutton.y);
           if (target == None)
             target = root;
           break;
@@ -485,27 +473,22 @@ scrot_sel_and_grab_image(void)
     count = select(fdsize, &fdset, NULL, NULL, NULL);
     if ((count < 0)
         && ((errno == ENOMEM) || (errno == EINVAL) || (errno == EBADF))) {
-      fprintf(stderr, "Connection to X display lost");
+      fprintf(stderr, "Connection to X display lost\n");
+      scrot_selection_destroy();
       exit(EXIT_FAILURE);
     }
   }
-  if (rect_w) {
-    XDrawRectangle(disp, root, gc, rect_x, rect_y, rect_w, rect_h);
-    XFlush(disp);
-  }
-  XUngrabPointer(disp, CurrentTime);
-  XUngrabKeyboard(disp, CurrentTime);
-  XFreeCursor(disp, cur_cross);
-  XFreeCursor(disp, cur_angle);
-  XFreeGC(disp, gc);
-  XSync(disp, True);
+  scrot_selection_draw();
 
-  if (opt.freeze == 1)
-      XUngrabServer(disp);
+  XUngrabKeyboard(disp, CurrentTime);
+
+  bool const isAreaSelect = (bool)(scrot_selection_get_rect().w > 5);
+
+  scrot_selection_destroy();
 
   if (done < 2) {
     scrot_do_delay();
-    if (rect_w > 5) {
+    if (isAreaSelect) {
       /* if a rect has been drawn, it's an area selection */
       rw = ev.xbutton.x - rx;
       rh = ev.xbutton.y - ry;
@@ -518,6 +501,10 @@ scrot_sel_and_grab_image(void)
         ry += rh;
         rh = 0 - rh;
       }
+
+      // Not record pointer if there is a selection area because it is busy on that
+      opt.pointer = 0;
+
     } else {
       /* else it's a window click */
       if (!scrot_get_geometry(target, &rx, &ry, &rw, &rh)) return NULL;
@@ -549,7 +536,7 @@ scrot_grab_autoselect(void)
 
 /* clip rectangle nicely */
 void
-scrot_nice_clip(int *rx, 
+scrot_nice_clip(int *rx,
   int *ry,
   int *rw,
   int *rh)
@@ -571,9 +558,9 @@ scrot_nice_clip(int *rx,
 /* get geometry of window and use that */
 int
 scrot_get_geometry(Window target,
-		   int *rx, 
-		   int *ry, 
-		   int *rw, 
+		   int *rx,
+		   int *ry,
+		   int *rw,
 		   int *rh)
 {
   Window child;
@@ -585,11 +572,11 @@ scrot_get_geometry(Window target,
     unsigned int d;
     int x;
     int status;
-    
+
     status = XGetGeometry(disp, target, &root, &x, &x, &d, &d, &d, &d);
     if (status != 0) {
       Window rt, *children, parent;
-      
+
       for (;;) {
 	/* Find window manager frame. */
 	status = XQueryTree(disp, target, &rt, &parent, &children, &d);
@@ -847,7 +834,8 @@ scrot_grab_stack_windows(void)
 
     XCompositeRedirectSubwindows(disp, root, CompositeRedirectAutomatic);
 
-    for (int i = 0; i < nitems_return; i++) {
+    int i;
+    for (i = 0; i < nitems_return; i++) {
 
         Window win = *((Window*)prop_return + i);
 
@@ -861,6 +849,10 @@ scrot_grab_stack_windows(void)
             continue;
         }
 
+        if (!scrot_match_window_class_name(win)) {
+            continue;
+        }
+
         ximage = XGetImage(disp, win, 0, 0, attr.width, attr.height, AllPlanes, ZPixmap);
 
         if (ximage == NULL) {
