@@ -11,6 +11,7 @@ Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel Lublin <daniel@lublin.se>
 Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
+Copyright 2020      daltomi <daltomi@disroot.org>-
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2020      spycapitan <spycapitan@protonmail.com>
 
@@ -37,6 +38,9 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "scrot.h"
 #include "options.h"
+#include <assert.h>
+
+#define MAX_LEN_WINDOW_CLASS_NAME  80
 
 static void scrot_parse_option_array(int argc, char **argv);
 scrotoptions opt;
@@ -51,9 +55,8 @@ init_parse_options(int argc, char **argv)
    opt.overwrite = 0;
    opt.line_style = LineSolid;
    opt.line_width = 1;
-   opt.line_color = NULL;
-   opt.display = NULL;
-   opt.stack = 0;
+   opt.line_opacity = 100;
+   opt.line_mode = LINE_MODE_CLASSIC;
 
    /* Parse the cmdline args */
    scrot_parse_option_array(argc, argv);
@@ -86,12 +89,14 @@ options_parse_required_number(char *str)
 static void
 options_parse_line(char *optarg)
 {
-   enum {STYLE = 0, WIDTH, COLOR };
+   enum {STYLE = 0, WIDTH, COLOR, OPACITY, MODE};
 
    char *const token[] = {
-      [STYLE] = "style",
-      [WIDTH] = "width",
-      [COLOR] = "color",
+      [STYLE]   = "style",
+      [WIDTH]   = "width",
+      [COLOR]   = "color",
+      [OPACITY] = "opacity",
+      [MODE]    = "mode",
       NULL
    };
 
@@ -103,7 +108,7 @@ options_parse_line(char *optarg)
 
          case STYLE:
 
-            if (value == NULL) {
+            if (value == NULL || *value == '\0') {
                fprintf(stderr, "Missing value for "
                      "suboption '%s'\n", token[STYLE]);
                exit(EXIT_FAILURE);
@@ -148,6 +153,47 @@ options_parse_line(char *optarg)
             opt.line_color = strdup(value);
 
             break;
+
+         case MODE:
+
+            if (value == NULL || *value == '\0') {
+               fprintf(stderr, "Missing value for "
+                     "suboption '%s'\n", token[MODE]);
+               exit(EXIT_FAILURE);
+            }
+
+            bool isValidMode = (bool)(0 == strncmp(value, LINE_MODE_CLASSIC, LINE_MODE_CLASSIC_LEN) );
+
+            isValidMode = isValidMode || (bool)(0 == strncmp(value, LINE_MODE_EDGE, LINE_MODE_EDGE_LEN));
+
+            if(isValidMode == false) {
+               fprintf(stderr, "Unknown value for "
+                     "suboption '%s': %s\n", token[MODE], value);
+               exit(EXIT_FAILURE);
+            }
+
+            opt.line_mode = strdup(value);
+
+            break;
+
+         case OPACITY:
+
+            if (value == NULL) {
+               fprintf(stderr, "Missing value for "
+                    "suboption '%s'\n", token[OPACITY]);
+               exit(EXIT_FAILURE);
+            }
+
+            opt.line_opacity = options_parse_required_number(value);
+
+            if (opt.line_opacity < 10 || opt.line_opacity > 100){
+               fprintf(stderr, "Value of the range (10..100) for "
+                     "suboption '%s': %d\n", token[OPACITY], opt.line_opacity);
+               exit(EXIT_FAILURE);
+             }
+
+            break;
+
          default:
             fprintf(stderr, "No match found for token: '%s'\n", value);
             exit(EXIT_FAILURE);
@@ -157,11 +203,22 @@ options_parse_line(char *optarg)
    } /* while */
 }
 
+static void options_parse_window_class_name(const char* window_class_name)
+{
+    assert(window_class_name != NULL);
+
+    if (window_class_name[0] != '\0' &&
+        strlen(window_class_name) < MAX_LEN_WINDOW_CLASS_NAME)
+    {
+        opt.window_class_name = strdup(window_class_name);
+    }
+}
+
 
 static void
 scrot_parse_option_array(int argc, char **argv)
 {
-   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:zn:l:D:k";
+   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:zn:l:D:kC:";
 
    static struct option lopts[] = {
       /* actions */
@@ -188,6 +245,7 @@ scrot_parse_option_array(int argc, char **argv)
       {"display", required_argument, 0, 'D'},
       {"note", required_argument, 0, 'n'},
       {"line", required_argument, 0, 'l'},
+      {"class", required_argument, 0, 'C'},
       {0, 0, 0, 0}
    };
    int optch = 0, cmdx = 0;
@@ -263,6 +321,9 @@ scrot_parse_option_array(int argc, char **argv)
         case 'k':
            opt.stack = 1;
         break;
+        case 'C':
+            options_parse_window_class_name(optarg);
+        break;
         case '?':
            exit(EXIT_FAILURE);
         default:
@@ -318,8 +379,9 @@ name_thumbnail(char *name)
       strcat(new_title, "-thumb");
       strcat(new_title, dot_pos);
    }
-   else
-      sprintf(new_title, "%s-thumb", name);
+   else {
+      snprintf(new_title, length, "%s-thumb", name);
+   }
 
    return new_title;
 }
@@ -422,6 +484,19 @@ void options_parse_note(char *optarg)
    scrot_note_new(opt.note);
 }
 
+/*
+Return:
+    0 : It does not match
+    1 : If it matches
+*/
+int options_cmp_window_class_name(const char* target_class_name)
+    {
+    assert(target_class_name != NULL);
+    assert(opt.window_class_name != NULL);
+    return !!(!strncmp(target_class_name, opt.window_class_name, MAX_LEN_WINDOW_CLASS_NAME - 1));
+}
+
+
 void
 show_version(void)
 {
@@ -476,6 +551,7 @@ show_usage(void)
            "                            See NOTE FORMAT\n"
            "  -k, --stack               Capture stack/overlaped windows and join them together.\n"
            "                            A running Composite Manager is needed.\n"
+           "  -C,  --class NAME         Window class name. Associative with options: -k\n"
            "\n" "  SPECIAL STRINGS\n"
            "  Both the --exec and filename parameters can take format specifiers\n"
            "  that are expanded by " SCROT_PACKAGE " when encountered.\n"
@@ -503,9 +579,13 @@ show_usage(void)
            "\n" "  SELECTION STYLE\n"
            "  When using --select you can indicate the style of the line with --line.\n"
            "  The following specifiers are recognised:\n"
-           "                  style=(solid,dash),width=(range 1 to 8),color=\"value\"\n"
-           "  The default style are:\n"
-           "                  style=solid,width=1\n"
+           "                  style=(solid,dash),width=(range 1 to 8),color=\"value\",\n"
+           "                  opacity=(range 10 to 100),mode=(edge,classic)\n"
+           "  The default style is:\n"
+           "                  mode=classic,style=solid,width=1,opacity=100\n"
+           "  Mode 'edge' ignore    : style, --freeze\n"
+           "  Mode 'classic' ignore : opacity\n\n"
+           "  The 'opacity' specifier is only effective if a Composite Manager is running.\n\n"
            "  For the color you can use a name or a hexdecimal value.\n"
            "                  color=\"red\" or color=\"#ff0000\"\n"
            "  Example:\n" "          " SCROT_PACKAGE
