@@ -5,6 +5,7 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
                                   scrot_sucks@linuxbrit.co.uk>
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2020      daltomi <daltomi@disroot.org>
 Copyright 2020      Jeroen Roovers <jer@gentoo.org>
 Copyright 2020      Hinigatsu <hinigatsu@protonmail.com>
 Copyright 2020      spycapitan <spycapitan@protonmail.com>
@@ -42,6 +43,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/cursorfont.h>
 #include <X11/extensions/Xfixes.h>
 #include <X11/extensions/Xcomposite.h>
+#include <X11/extensions/shape.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -59,6 +61,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <signal.h>
 #include <sys/wait.h>
 #include <giblib/giblib.h>
+#include <stdbool.h>
 
 
 #include "scrot_config.h"
@@ -66,6 +69,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "getopt.h"
 #include "debug.h"
 #include "note.h"
+#include "scrot_selection.h"
 
 #ifndef __GNUC__
 # define __attribute__(x)
@@ -88,6 +92,7 @@ Imlib_Image scrot_grab_autoselect(void);
 void scrot_sel_area(int *x, int *y, int *w, int *h);
 void scrot_nice_clip(int *rx, int *ry, int *rw, int *rh);
 int scrot_get_geometry(Window target, int *rx, int *ry, int *rw, int *rh);
+int scrot_match_window_class_name(Window target);
 Window scrot_get_window(Display *display,Window window,int x,int y);
 Window scrot_get_client_window(Display * display, Window target);
 Window scrot_find_window_by_property(Display * display, const Window window,
@@ -103,7 +108,7 @@ void scrot_grab_mouse_pointer(const Imlib_Image image,
 		const int ix_off, const int iy_off);
 
 void scrot_check_if_overwrite_file(char **filename);
-char *scrot_have_file_extension(char *filename);
+size_t scrot_have_file_extension(char const *filename, char **ext);
 
 /* Imlib stuff */
 extern Display *disp;
